import { world, system, EquipmentSlot, EntityEquippableComponent } from "@minecraft/server"
import { damageItem } from "./lib/durability";

const calculateYDifference = (a, b) => a.y - b.y;

function getDistance3D(p1, p2) {
  return Math.sqrt(
    Math.pow(p2.x - p1.x, 2) +
    Math.pow(p2.y - p1.y, 2) +
    Math.pow(p2.z - p1.z, 2)
  );
}

system.runInterval(() => {
  for (const player of world.getPlayers()) {
    const view = player.getViewDirection()
    const loc = player.location
    if (!player.isSneaking) return
    const equipment = player.getComponent("minecraft:equippable");
    const mainHand = equipment?.getEquipmentSlot("Mainhand")?.getItem();
    const offHand = equipment?.getEquipmentSlot("Offhand")?.getItem()
    let shieldHand = null;
    let shieldItem = null;
    if (mainHand?.hasTag("template:shield") && offHand?.typeId !== "minecraft:shield" && !offHand?.hasTag("template:shield")) {
      shieldHand = "Mainhand";
      shieldItem = mainHand;
    } else if (offHand?.hasTag("template:shield") && mainHand?.typeId !== "minecraft:shield" && !mainHand?.hasTag("template:shield")) {
      shieldHand = "Offhand";
      shieldItem = offHand;
    }
    if (!shieldHand || !shieldItem) return;
    const allEntities = player.dimension.getEntities()
    for (const entity of allEntities) {
      const isProjectile = entity?.hasComponent("minecraft:projectile")
      if (isProjectile && getDistance3D({ x: loc.x, y: loc.y + 0.5, z: loc.z }, entity.location) < 2) {
        if (player.getGameMode() === "survival") {
          const newItem = damageItem(shieldItem);
          equipment.setEquipment(shieldHand, newItem);
          world.playSound(newItem ? "item.shield.block" : "random.break", player.location);
        }
        entity.applyImpulse({ x: view.x * 1.5, y: view.y * 1.5, z: view.z * 1.5 });
		entity.remove()
      }
    }
  }
}, 0);

world.afterEvents.entityHitEntity.subscribe(({ hitEntity: player, damagingEntity: damager }) => {
  if (player.typeId !== "minecraft:player" || !player.isSneaking) return;
  const health = player.getComponent("minecraft:health");
  const equipment = player.getComponent("minecraft:equippable");
  const mainHand = equipment?.getEquipmentSlot("Mainhand")?.getItem();
  const offHand = equipment?.getEquipmentSlot("Offhand")?.getItem();
  const damagerOnFire = damager.hasComponent("onfire")
  let shieldHand = null;
  let shieldItem = null;
  if (mainHand?.hasTag("template:shield") && offHand?.typeId !== "minecraft:shield" && !offHand?.hasTag("template:shield")) {
    shieldHand = "Mainhand";
    shieldItem = mainHand;
  } else if (offHand?.hasTag("template:shield") && mainHand?.typeId !== "minecraft:shield" && !mainHand?.hasTag("template:shield")) {
    shieldHand = "Offhand";
    shieldItem = offHand;
  }
  if (!shieldHand || !shieldItem) return;
  const rotationDifference = calculateYDifference(player.getRotation(), damager.getRotation());
  if (rotationDifference <= 90 && rotationDifference >= -90) return;
  system.runTimeout(() => {
    if (player.getGameMode() === "survival") {
      const newItem = damageItem(shieldItem);
      equipment.setEquipment(shieldHand, newItem);
      world.playSound(newItem ? "" : "random.break", player.location);
    }
    if (damagerOnFire) player.extinguishFire()
    const damage = player["damage"] ?? 0;
    health.setCurrentValue(health.currentValue + damage);
  });
  const direction = {
    x: damager.location.x - player.location.x,
    y: damager.location.y - player.location.y,
    z: damager.location.z - player.location.z,
  };
  damager.applyKnockback(direction.x, direction.z, 1.3+slime, 0.52);
  player.teleport(player.location, { rotation: player.getRotation() });
});

world.afterEvents.entityHurt.subscribe(({ hurtEntity: player, damage }) => {
  if (player.typeId !== "minecraft:player") return

  player["damage"] = damage
  system.runTimeout(() => player["damage"] = null, 2)
});

world.afterEvents.playerBreakBlock.subscribe(({ player, itemStackAfterBreak: item }) => {
  if (item?.hasTag("template:durability")) {
    const equipment = player.getComponent("minecraft:equippable");
    const newItem = damageItem(item);

    equipment.setEquipment("Mainhand", newItem);
    world.playSound(newItem ? "" : "random.break", player.location);
  }
});

world.afterEvents.entityHitEntity.subscribe(({ hitEntity: entity, damagingEntity: damager }) => {
  if (damager.typeId !== "minecraft:player") return

  const equipment = damager.getComponent("minecraft:equippable");
  const item = equipment?.getEquipmentSlot("Mainhand")?.getItem();

  if (item?.hasTag("template:durability")) {
    const equipment = damager.getComponent("minecraft:equippable");
    const newItem = damageItem(item);

    equipment.setEquipment("Mainhand", newItem);
    world.playSound(newItem ? "" : "random.break", damager.location);
  }
})

world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
  itemComponentRegistry.registerCustomComponent("template:unbreakable", {
    onBeforeDurabilityDamage: (e) => {
      e.durabilityDamage = 0
    }
  })
})

system.runInterval(() =>{
	for(let p of world.getPlayers()){
		let equip = p.getComponent(EntityEquippableComponent.componentId);
		try{
			let item = equip?.getEquipment(EquipmentSlot.Mainhand);
			if(item.typeId.includes("stm:") && item.typeId.includes("_shield") && !p.hasTag("offhand") && p.isSneaking){
				p.runCommandAsync(`playanimation @s animation.player.custom_shield_block_main_hand root 100`)
				p.addTag("mainhand")
			}
			if(item.typeId.includes("stm:glowstone_shield")){
				p.runCommandAsync("execute as @s positioned ~~1~ run function light13")
				p.addTag("glowstone_main")
			}
			if(item.typeId.includes("stm:slime_shield")){
				slime = 3;
			}
			if(!item.typeId.includes("stm:slime_shield")){
				slime = 0;
			}
			if(!item.typeId.includes("stm:") || !item.typeId.includes("_shield") || p.hasTag("offhand") || !p.isSneaking){
				if(p.hasTag("mainhand")){
					p.runCommandAsync("playanimation @s animation.player.custom_shield_block_main_hand root")
					p.removeTag("mainhand")
				}
			}
			if(!item.typeId.includes("stm:glowstone_shield")){
				if(p.hasTag("glowstone_main")){
					p.runCommandAsync("function no_light")
					p.removeTag("glowstone_main")
				}
			}
		}catch(e){
			if(p.hasTag("mainhand")){
				p.runCommandAsync("playanimation @s animation.player.custom_shield_block_main_hand root")
				p.removeTag("mainhand")
				slime = 0;
			}
			if(p.hasTag("glowstone_main")){
				p.runCommandAsync("function no_light")
				p.removeTag("glowstone_main")
			}
		}
		try{
			let off = equip?.getEquipment(EquipmentSlot.Offhand);
			if(off.typeId.includes("stm:") && off.typeId.includes("_shield") && p.isSneaking){
				p.runCommandAsync(`playanimation @s animation.player.custom_shield_block_off_hand root 100`)
				p.addTag("offhand")
			}
			if(off.typeId.includes("stm:glowstone_shield")){
				p.runCommandAsync("execute as @s positioned ~~1~ run function light13")
				p.addTag("glowstone_off")
			}
			if(!off.typeId.includes("stm:") || !off.typeId.includes("_shield") || !p.isSneaking){
				if(p.hasTag("offhand")){
					p.runCommandAsync("playanimation @s animation.player.custom_shield_block_off_hand root")
					p.removeTag("offhand")
				}
			}
			if(!off.typeId.includes("stm:glowstone_shield")){
				if(p.hasTag("glowstone_off")){
					p.runCommandAsync("function no_light")
					p.removeTag("glowstone_off")
				}
			}
		}catch(e){
			if(p.hasTag("offhand")){
				p.runCommandAsync("playanimation @s animation.player.custom_shield_block_off_hand root")
				p.removeTag("offhand")
			}
			if(p.hasTag("glowstone_off")){
				p.runCommandAsync("function no_light")
				p.removeTag("glowstone_off")
			}

		}
	}
})

export function addEffect(player, effectType, duration, level, particles) {
	return player.addEffect(effectType, duration*2, {amplifier: level, showParticles: particles}) ?? 0;
}
export function removeEffect(player, effectType) {
	return player.removeEffect(effectType) ?? 0;
}

world.afterEvents.playerSpawn.subscribe(spawn =>{
	let p = spawn.player;
	if(p.typeId == "minecraft:player"){
		p.runCommandAsync("gamerule sendcommandfeedback false")
	}
})
let slime = 0;
world.afterEvents.entityHurt.subscribe(hurt =>{
	let eHurt = hurt.hurtEntity;
	let cause = hurt.damageSource.cause;
	let damaging = hurt.damageSource.damagingEntity;
	if(eHurt.typeId == "minecraft:player"){
		let equip = eHurt.getComponent(EntityEquippableComponent.componentId);
		let damage = Math.round(hurt.damage);
		try{
			let item = equip?.getEquipment(EquipmentSlot.Mainhand);
			let mainDamage = item.getComponent("durability").damage;
			let maxDurability = item.getComponent("durability").maxDurability;
			if(!eHurt.hasTag("offhand") && eHurt.isSneaking){
				if(item.typeId.includes("stm:magma_shield")){
					damaging.setOnFire(5, true)
					eHurt.runCommandAsync("playsound break.calcite @s")
				}
				if(item.typeId.includes("stm:dripstone_shield")){
					damaging.runCommandAsync(`damage @s ${damage+3}`)
					eHurt.runCommandAsync("playsound break.pointed_dripstone @s")
				}
				if(item.typeId.includes("stm:slime_shield")){
					eHurt.runCommandAsync("playsound hit.slime @s")
				}
				if(item.typeId.includes("stm:mushroom_shield")){
					eHurt.runCommandAsync("playsound dig.wood @s")
					addEffect(damaging, "poison", 30, 3, true)
				}
				if(item.typeId.includes("stm:glowstone_shield")){
					eHurt.runCommandAsync("playsound random.glass @s")
				}
				if(item.typeId.includes("stm:wither_shield")){
					eHurt.runCommandAsync("playsound break.pointed_dripstone @s")
					addEffect(damaging, "wither", 30, 3, true)
				}
				if(item.typeId.includes("stm:ender_shield")){
					eHurt.runCommandAsync("playsound break.pointed_dripstone @s")
					system.runTimeout(() =>{
						damaging.runCommandAsync("tp ~~10~")
						eHurt.runCommandAsync("playsound mob.shulker.teleport @s")
					}, 3)
				}
				if(item.typeId.includes("stm:shulker_shield")){
					eHurt.runCommandAsync("playsound mob.shulker.close @s")
					addEffect(damaging, "levitation", 30, 5, true)
				}
			}
		}
		catch(e){
		}
		try{
			let off = equip?.getEquipment(EquipmentSlot.Offhand);
			let offDamage = off.getComponent("durability").damage;
			let offmaxDurability = off.getComponent("durability").maxDurability;
			if(eHurt.isSneaking){
				if(off.typeId.includes("stm:magma_shield")){
					damaging.setOnFire(5, true)
					eHurt.runCommandAsync("playsound break.calcite @s")
				}
				if(off.typeId.includes("stm:dripstone_shield")){
					damaging.runCommandAsync(`damage @s ${damage+1}`)
					eHurt.runCommandAsync("playsound break.pointed_dripstone @s")
				}
				if(off.typeId.includes("stm:slime_shield")){
					slime = 3;
					eHurt.runCommandAsync("playsound hit.slime @s")
				}
				if(off.typeId.includes("stm:mushroom_shield")){
					eHurt.runCommandAsync("playsound dig.wood @s")
					addEffect(damaging, "poison", 30, 3, true)
				}
				if(off.typeId.includes("stm:glowstone_shield")){
					eHurt.runCommandAsync("playsound random.glass @s")
				}
				if(off.typeId.includes("stm:wither_shield")){
					eHurt.runCommandAsync("playsound break.pointed_dripstone @s")
					addEffect(damaging, "wither", 30, 3, true)
				}
				if(off.typeId.includes("stm:ender_shield")){
					eHurt.runCommandAsync("playsound break.pointed_dripstone @s")
					system.runTimeout(() =>{
						damaging.runCommandAsync("tp ~~10~")
						eHurt.runCommandAsync("playsound mob.shulker.teleport @s")
					}, 3)
				}
				if(off.typeId.includes("stm:shulker_shield")){
					eHurt.runCommandAsync("playsound mob.shulker.close @s")
					addEffect(damaging, "levitation", 30, 5, true)
				}
			}
		}
		catch(e){
		}
	}
})